%{
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <mem.h>

#include "consthash.h"
#include "types.h"
#include "binfuncs.h"


extern int yyparse( void );
extern int yylex (void);
extern int yylex_destroy (void);
extern int yylineno;


static expr expr_result = NULL;
static int yyerrors   = 0;
int verbose = 0;		/* Boolean */



void yyerror(const char *str)
{
        fprintf(stderr,"line %d: error: %s\n", yylineno, str);
	yyerrors++;
}
 
int yywrap( void )
{
        return 1;
} 

int main( int argc, char **argv )
{
	int opt;

        while ((opt = getopt(argc, argv, "ve")) != -1) {
		switch (opt) {
		case 'v':
			verbose = 1;
			break;
		default: /* '?' */
			fprintf(stderr, "Usage: %s [-v] [arg1 [arg2..]]\n",
				argv[0]);
			fprintf(stderr, "       -v: verbose\n" );
			exit(1);
		}
	}

	int i, j;
	for( i=optind, j=1; i<argc; i++, j++ )
	{
		argv[j] = argv[i];
	}
	argc -= optind-1;

	init_consthash( argc, argv );
        if( yyparse() == 0 && yyerrors == 0 )
	{
		printf( "parsing succeeded\n" );
		printf( "result: " );
		print_expr( stdout, expr_result );
		printf( "\n" );
		free_expr( expr_result );
	} else
	{
		fprintf( stderr, "parsing failed with %d errors\n", yyerrors );
	}
	destroy_consthash();
	yylex_destroy();
	exit(0);
	return 0;
} 
%}

%union {
	int n;
	char *s;
	expr e;
}

%token PLUS MINUS MUL DIV MOD OPEN CLOSE TOKERR
%token <n> NUMBER
%token <s> IDENT

%type <e> expr term factor

%start oneexpr

%%
oneexpr      :  expr            { expr_result = $1; }
             ;

expr         : expr PLUS term   { $$ = plus( $1, $3 ); }
             | expr MINUS term  { $$ = minus( $1, $3 ); }
             | term
             ;

term         : term MUL factor  { $$ = times( $1, $3 ); }
             | term DIV factor  { $$ = divide( $1, $3 ); }
             | term MOD factor  { $$ = mod( $1, $3 ); }
             | factor
             ;

factor       : NUMBER           { $$ = expr_num($1); }
	     | IDENT            { $$ = expr_id($1); }
             | OPEN expr CLOSE  { $$ = $2; }
             ;
